using System;
using System.Data;
using System.Data.SqlClient;
using System.IO;
using System.Net;
using ITHit.WebDAV.Logger;
using ITHit.WebDAV.Server;
using ITHit.WebDAV.Server.Response;

namespace WebDAVServer.Class2SqlListener
{
    public class Resource : HierarchyItem, IResource
    {
        public Resource(Engine engine, int ID, string name, string path, DateTime created, DateTime modified)
            : base(engine, ID, name, path, created, modified)
        {
        }

        #region IResource Members

        public string ContentType
        {
            get
            {
                string contentType = "";

                SqlConnection conn = new SqlConnection(connStr);
                SqlCommand cmd;
                SqlDataReader reader = null;
                conn.Open();

                try
                {
                    cmd = conn.CreateCommand();
                    cmd.CommandText = "SELECT ContentType FROM Repository WHERE ID = @ID";
                    cmd.Parameters.Add("@ID", SqlDbType.Int).Value = ID;
                    reader = cmd.ExecuteReader();
                    reader.Read();
                    if (!reader.IsDBNull(reader.GetOrdinal("ContentType")))
                        contentType = reader.GetString(reader.GetOrdinal("ContentType"));
                }
                finally
                {
                    if (reader != null) reader.Close();
                    conn.Close();
                }

                if (contentType == "")
                    contentType = "application/octet-stream";
                return contentType;
            }
        }

        public long ContentLength
        {
            get
            {
                long length = 0;

                SqlConnection conn = new SqlConnection(connStr);
                SqlCommand cmd;
                SqlDataReader reader = null;
                conn.Open();

                try
                {
                    cmd = conn.CreateCommand();
                    cmd.CommandText = "SELECT Content FROM Repository WHERE ID = @ID";
                    cmd.Parameters.Add("@ID", SqlDbType.Int).Value = ID;

                    reader = cmd.ExecuteReader(CommandBehavior.SequentialAccess);
                    reader.Read();
                    if (!reader.IsDBNull(reader.GetOrdinal("Content")))
                        length = reader.GetBytes(reader.GetOrdinal("Content"), 0, null, 0, 0);
                }
                finally
                {
                    if (reader != null) reader.Close();
                    conn.Close();
                }

                return length;
            }
        }

        public WebDAVResponse WriteToStream(Stream output, long byteStart, long count)
        {
            //System.Web.HttpContext.Current.Server.ScriptTimeout = 1200;

            SqlConnection conn = new SqlConnection(connStr);
            SqlCommand cmd;
            SqlDataReader reader = null;
            conn.Open();

            try
            {
                cmd = conn.CreateCommand();
                cmd.CommandText = "SELECT Content FROM Repository WHERE ID = @ID";
                cmd.Parameters.Add("@ID", SqlDbType.Int).Value = ID;

                reader = cmd.ExecuteReader(CommandBehavior.SequentialAccess);
                reader.Read();

                long bufSize = 1048576; // 1Mb
                byte[] buf = new byte[bufSize];
                long retval;
                long startIndex = byteStart;

                while ((retval = reader.GetBytes(reader.GetOrdinal("Content"),
                                                 startIndex, buf, 0, (int) (count > bufSize ? bufSize : count))) > 0)
                {
                    try
                    {
                        output.Write(buf, 0, (int) retval);
                    }
                    catch (HttpListenerException)
                    {
                        // The remote host closed the connection
                        // client closed connection
                        // 1. ErrorCode=1229. An operation was attempted on a nonexistent network connection.
                        // 2. ErrorCode=64. The specified network name is no longer available.
                        return new OkResponse();
                    }

                    startIndex += retval;
                    count -= retval;
                }
            }
            finally
            {
                if (reader != null) reader.Close();
                conn.Close();
            }

            return new OkResponse();
        }

        public WebDAVResponse SaveFromStream(Stream content, string contentType)
        {
            if (!ClientHasToken)
                return new LockedResponse();

            WebDAVResponse resp;

            SqlConnection conn = new SqlConnection(connStr);
            SqlCommand cmd;
            SqlTransaction trans = null;
            conn.Open();

            try
            {
                trans = conn.BeginTransaction();
                cmd = conn.CreateCommand();
                cmd.Transaction = trans;
                if (WriteContent(trans, content, contentType) > 0)
                {
                    resp = new OkResponse();
                }
                else
                {
                    resp = new NoContentResponse();
                }

                trans.Commit();
            }
            catch (Exception ex)
            {
                if (trans != null) trans.Rollback();
                FileLogger.WriteMessage(ex.ToString(), LogLevel.Fatal);
                resp = new ServerErrorResponse();
            }
            finally
            {
                conn.Close();
            }

            return resp;
        }

        public int WriteContent(SqlTransaction trans, Stream content, string contentType)
        {
            //System.Web.HttpContext.Current.Server.ScriptTimeout = 1200;

            SqlCommand cmd = trans.Connection.CreateCommand();
            cmd.Transaction = trans;
            int bytes = 0;

            if (content != Stream.Null)
            {
                int bufSize = 1048576; // 1Mb

                cmd.CommandText = "UPDATE Repository SET"
                                  + " Modified = GETUTCDATE(),"
                                  + " Content = NULL,"
                                  + " ContentType = @ContentType"
                                  + " WHERE ID = @ID;"
                                  + "SELECT @Pointer = TEXTPTR(Content) FROM Repository WHERE ID = @ID";

                if (contentType != null)
                    cmd.Parameters.Add("@ContentType", SqlDbType.NVarChar).Value = contentType;
                else
                    cmd.Parameters.Add("@ContentType", SqlDbType.NVarChar).Value = DBNull.Value;
                cmd.Parameters.Add("@ID", SqlDbType.Int).Value = ID;

                SqlParameter ptrParm = cmd.Parameters.Add("@Pointer", SqlDbType.Binary, 16);
                ptrParm.Direction = ParameterDirection.Output;

                cmd.ExecuteNonQuery();

                cmd.CommandText = "UPDATETEXT Repository.Content @Pointer @Offset NULL @Bytes";

                cmd.Parameters.Clear();
                cmd.Parameters.Add("@Pointer", SqlDbType.Binary, 16).Value = ptrParm.Value;
                SqlParameter offsetParm = cmd.Parameters.Add("@Offset", SqlDbType.Int);
                SqlParameter bytesParm = cmd.Parameters.Add("@Bytes", SqlDbType.Image, bufSize);

                BinaryReader br = new BinaryReader(content);

                byte[] buf = br.ReadBytes(bufSize);
                int offset_ctr = 0;

                while (buf.Length > 0)
                {
                    bytesParm.Value = buf;
                    offsetParm.Value = offset_ctr;
                    cmd.ExecuteNonQuery();
                    offset_ctr += buf.Length;
                    buf = br.ReadBytes(bufSize);
                }

                bytes = offset_ctr;

                br.Close();
            }
            else
            {
                cmd.CommandText = "UPDATE Repository SET"
                                  + " Modified = GETUTCDATE(),"
                                  + " Content = NULL,"
                                  + " ContentType = @ContentType"
                                  + " WHERE ID = @ID";

                if (contentType != null)
                    cmd.Parameters.Add("@ContentType", SqlDbType.NVarChar).Value = contentType;
                else
                    cmd.Parameters.Add("@ContentType", SqlDbType.NVarChar).Value = DBNull.Value;
                cmd.Parameters.Add("@ID", SqlDbType.Int).Value = ID;

                cmd.ExecuteNonQuery();
            }

            return bytes;
        }

        #endregion // IResource Members

        public override WebDAVResponse CopyTo(IFolder folder, string destName, bool deep)
        {
            Folder destFolder = folder as Folder;
            if (destFolder == null)
                return new ConflictResponse();
            if (!destFolder.ClientHasToken)
                return new LockedResponse();

            HierarchyItem destItem = destFolder.FindChild(destName);
            if (destItem != null)
            {
                if (DelWhenReplace && destItem is IFolder)
                {
                    Folder destItemF = destItem as Folder;
                    if (!destItemF.ClientHasTokenForTree())
                        return new LockedResponse();
                    destItemF.RemoveTree();
                    CopyThisItem(destFolder, null, destName);
                    return new NoContentResponse();
                }
                else if (destItem is IResource)
                {
                    if (destItem.ClientHasToken)
                    {
                        CopyThisItem(null, destItem, destName);
                        return new NoContentResponse();
                    }
                    else // destination is locked
                        return new LockedResponse();
                }
                else
                    return new ConflictResponse();
            }
            else
            {
                CopyThisItem(destFolder, null, destName);
                return new CreatedResponse();
            }
        }

        public override WebDAVResponse Delete()
        {
            Folder parent = Parent as Folder;
            if (parent == null)
                return new ConflictResponse();
            if (!parent.ClientHasToken)
                return new LockedResponse();

            if (!ClientHasToken)
                return new LockedResponse();

            DeleteThisItem();
            return new NoContentResponse();
        }

        public override WebDAVResponse MoveTo(IFolder folder, string destName)
        {
            Folder destFolder = folder as Folder;
            if (destFolder == null)
                return new ConflictResponse();

            Folder parent = Parent as Folder;
            if (parent == null)
                return new ConflictResponse();

            if (!ClientHasToken || !destFolder.ClientHasToken || !parent.ClientHasToken)
                return new LockedResponse();

            HierarchyItem destItem = destFolder.FindChild(destName);
            if (destItem != null)
            {
                if (DelWhenReplace && destItem is IFolder)
                {
                    Folder destItemF = destItem as Folder;
                    if (!destItemF.ClientHasTokenForTree())
                        return new LockedResponse();
                    destItemF.RemoveTree();
                    MoveThisItem(destFolder, destName, parent);
                    return new NoContentResponse();
                }
                else if (destItem is IResource)
                {
                    WebDAVResponse delResp = destItem.Delete();
                    if (!delResp.IsSuccess)
                        return delResp;
                    MoveThisItem(destFolder, destName, parent);
                    return new NoContentResponse();
                }
                else
                    return new ConflictResponse();
            }
            else
            {
                MoveThisItem(destFolder, destName, parent);
                return new CreatedResponse();
            }
        }
    }
}